#include <sstream>

#include "../InputModule/Input.h"

#include "../GfxModule/GfxModule.h"

#include "../SoundModule/SoundModuleWrapper.h"
#include "../SoundModule/ChunkWrapper.h"
#include "../SoundModule/SfxManager.h"

#include "../StateModule/GameStateManager.h"
#include "../ToolKit/SdlResize.h"
#include "./MainGame.h"
#include "./MusicState.h"

using namespace GfxModule;
using namespace SoundModule;
using namespace InputModule;
using namespace GameStateModule;

using namespace std;

void MainGameState::PushInitialise(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    // Load Ken into the surface manager
    SurfaceWrapper * ASurfaceWrapper = AGfxModule->GetSurfaceManager()->CreateSurface("./Ken.png");
    _KenSprite = AGfxModule->GetSpriteManager()->GetFreeSprite();
    _KenSprite->SetSurfaceWrapper(ASurfaceWrapper);
    _KenSprite->SetSpriteProperties(
        40,     // FrameWidth
        70,     // FrameHeight
        6,      // Number Of Frames
        3,      // Frame Delay
        true    // Loop animation
    );
    _KenSprite->SetXPosition(100);
    _KenSprite->SetYPosition(100);
    _KenSprite->SetPriority(0);

    ASurfaceWrapper = AGfxModule->GetSurfaceManager()->CreateSurface("./Layer4.png");
    _MarioBG = AGfxModule->GetBGManager()->GetBGObject(0);
    _MarioBG->SetSurfaceWrapper(ASurfaceWrapper);

    ASurfaceWrapper = AGfxModule->GetSurfaceManager()->CreateSurface("./Layer3.png");
    _CircleBG = AGfxModule->GetBGManager()->GetBGObject(1);
    _CircleBG->SetSurfaceWrapper(ASurfaceWrapper);

    ASurfaceWrapper = AGfxModule->GetSurfaceManager()->CreateSurface("./Layer1.png");
    _HoleBG = AGfxModule->GetBGManager()->GetBGObject(2);
    _HoleBG->SetSurfaceWrapper(ASurfaceWrapper);

    // Load the font
    _CourierNewFont = AGfxModule->GetFontManager()->CreateFont("./Courier_New_Bold.ttf", 12);

    // Load the sound effects
    _DrumRollSound = ASoundModule->GetSfxManager()->CreateSfx("./drum_roll.wav");
    _GlassBreakSound = ASoundModule->GetSfxManager()->CreateSfx("./glass_breaking.wav");
}

//! Pure virtual. Cleanup up the state when it is pushed then further into the stack
void MainGameState::PushCleanup(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    _KenSprite->PauseAnimation();
}

//! Pure virtual. When the state is popped up to the top of the stack
void MainGameState::PopInitialise(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    _KenSprite->ResumeAnimation();
}

//! Pure virtual. When the state is popped off the stack
void MainGameState::PopCleanup(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    // Halt all Sound
	ASoundModule->GetSfxManager()->StopAllChannels();
    // Free everything
    AGfxModule->GetBGManager()->ClearAllBGObjects();
    AGfxModule->GetSpriteManager()->FreeAllSprites();
    AGfxModule->GetSurfaceManager()->FreeAllSurfaces();
    AGfxModule->GetFontManager()->FreeAllFonts();
    ASoundModule->GetSfxManager()->FreeAllSfx();
}

//! Pure virtual. Gets called once per frame when it is on top of the stack
void MainGameState::Update(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule,
    Input* AInputModule,
    GameStateManager* AGameStateManager
)
{
    // Update the FPS counter
    _FpsCounter.Update();

    // Print the FPS count in the top right in white
    unsigned int CurrentFps = _FpsCounter.GetFpsCount();
    ostringstream FpsOStringStream;
    FpsOStringStream << "FPS: " << CurrentFps;
    AGfxModule->GetFontManager()->WriteText(_CourierNewFont, 1, 1, FpsOStringStream.str(), 255, 255, 255);

    // Update the input
    AInputModule->UpdateInputState();
    unsigned int CurrentInputState = AInputModule->GetCurrentInputState();
    unsigned int PreviousInputState = AInputModule->GetPreviousInputState();

    int MoveY = 0, MoveX = 0;
    if ( CurrentInputState & LEFT )
    {
        MoveX = 12;
    }

    if ( CurrentInputState & RIGHT )
    {
        MoveX = -12;
    }

    if ( CurrentInputState & UP )
    {
        MoveY = 12;
    }

    if ( CurrentInputState & DOWN )
    {
        MoveY = -12;
    }

    if ( CurrentInputState & BUTTON_A )
    {
        if ( !(PreviousInputState & BUTTON_A) )
            _MarioBG->ScaleBG(320, 320, BOX_FILTER);
    }

    if ( CurrentInputState & BUTTON_Y )
    {
        if ( !(PreviousInputState & BUTTON_Y) )
            AGameStateManager->Push(MusicState::GetSingletonPtr() );
    }

    if ( CurrentInputState & BUTTON_B )
    {
        if ( !(PreviousInputState & BUTTON_B) )
            _KenSprite->ScaleSprite(2.0f, BOX_FILTER);
    }

    if ( CurrentInputState & BUTTON_X )
    {
        if ( !(PreviousInputState & BUTTON_X) )
            _KenSprite->ScaleSprite(0.5f, BOX_FILTER);
    }

    if ( CurrentInputState & BUTTON_L )
    {
		if ( !(PreviousInputState & BUTTON_L) )
			ASoundModule->GetSfxManager()->PlaySfx(_GlassBreakSound);
    }

    if ( CurrentInputState & BUTTON_R )
    {
		if ( !(PreviousInputState & BUTTON_R) )
			ASoundModule->GetSfxManager()->PlaySfx(_DrumRollSound);
    }

    if (CurrentInputState & BUTTON_SELECT )
    {
		ASoundModule->GetSfxManager()->StopAllChannels();
    }

    if (CurrentInputState & BUTTON_VOLDOWN )
    {
        ASoundModule->GetSfxManager()->VolumeDown();
    }

    if (CurrentInputState & BUTTON_VOLUP )
    {
        ASoundModule->GetSfxManager()->VolumeUp();
    }

    if ( CurrentInputState & BUTTON_JOYCLICK )
    {
        // Quit the game
        if ( !(PreviousInputState & BUTTON_JOYCLICK) )
           AGameStateManager->Pop();
    }

    // Move the backgrounds
    _MarioBG->MoveBG(MoveX/3, MoveY/3);
    _CircleBG->MoveBG(MoveX, MoveY);

    // Update the FPS counter
    _FpsCounter.Update();

    // Print the FPS count in the top right in white
    unsigned int CurrentFps = _FpsCounter.GetFpsCount();
    ostringstream FpsOStringStream;
    FpsOStringStream << CurrentFps;
    AGfxModule->GetFontManager()->WriteText(_CourierNewFont, 1, 1, FpsOStringStream.str(), 255, 255, 255);
}
