#include <SDL/SDL.h>
#include "../Logger/Logger.h"
#include "./Input.h"

using namespace LogModule;
using namespace InputModule;

#ifdef PC
const unsigned int
	GP2X_JOY_UP        	      = SDLK_w,
	GP2X_JOY_DOWN             = SDLK_x,
	GP2X_JOY_LEFT             = SDLK_a,
	GP2X_JOY_RIGHT            = SDLK_d,
	GP2X_JOY_UPLEFT           = SDLK_q,
	GP2X_JOY_UPRIGHT          = SDLK_e,
	GP2X_JOY_DOWNLEFT         = SDLK_z,
	GP2X_JOY_DOWNRIGHT		  = SDLK_c,
	GP2X_JOY_CLICK        	  = SDLK_s,
	GP2X_BUTTON_A             = SDLK_i,
	GP2X_BUTTON_B             = SDLK_l,
	GP2X_BUTTON_X             = SDLK_k,
	GP2X_BUTTON_Y             = SDLK_o,
	GP2X_BUTTON_L             = SDLK_u,
	GP2X_BUTTON_R             = SDLK_p,
	GP2X_BUTTON_START         = SDLK_y,
	GP2X_BUTTON_SELECT        = SDLK_t,
	GP2X_BUTTON_VOLUP         = SDLK_h,
	GP2X_BUTTON_VOLDOWN       = SDLK_g;
#endif // PC

#ifdef GP2X
const unsigned int
	GP2X_JOY_UP        	      = (0),
	GP2X_JOY_DOWN             = (4),
	GP2X_JOY_LEFT             = (2),
	GP2X_JOY_RIGHT            = (6),
	GP2X_JOY_UPLEFT           = (1),
	GP2X_JOY_UPRIGHT          = (7),
	GP2X_JOY_DOWNLEFT         = (3),
	GP2X_JOY_DOWNRIGHT		  = (5),
	GP2X_JOY_CLICK        	  = (18),
	GP2X_BUTTON_A             = (12),
	GP2X_BUTTON_B             = (13),
	GP2X_BUTTON_X             = (15),
	GP2X_BUTTON_Y             = (14),
	GP2X_BUTTON_L             = (10),
	GP2X_BUTTON_R             = (11),
	GP2X_BUTTON_START         = (8),
	GP2X_BUTTON_SELECT        = (9),
	GP2X_BUTTON_VOLUP         = (16),
	GP2X_BUTTON_VOLDOWN       = (17);
#endif // GP2X

Input::Input(void) : _CurrentInputState(0), _PreviousInputState(0), _KillFlag(false)
{
	ErrLog << "<< Creating Input Module >>\n";
	#ifdef GP2X
	// Hide the cursor
	ErrLog << "Hid the mouse cursor\n";
	SDL_ShowCursor(SDL_DISABLE);
	// Open the joystick for the GP2X
	ErrLog << "Opening Joystick port 0 for GP2X\n";
	if ( SDL_InitSubSystem(SDL_INIT_JOYSTICK) < 0 )
    {
    	ErrLog << "Unable to init SDL_Joystick: " << SDL_GetError() << "\n";
    }
    else
    {
    	// Open the joystick
    	ErrLog << "Opened Joystick port 0 for GP2X\n";
    	SDL_JoystickOpen(0);
    }
	#endif // GP2X
	ErrLog << "<< Created Input Module>>\n";
}

Input::~Input(void)
{
	ErrLog << "<< Destroying the Input Module >>\n";
	ErrLog << "<< Destroyed the Input Module >>\n";
}

void Input::UpdateInputState(void)
{
	// Store the current state to the previous buffer
	_PreviousInputState = _CurrentInputState;

	// Clear current state
	//_CurrentInputState = 0;

	// message processing loop
	SDL_Event Event;
	while (SDL_PollEvent(&Event))
	{
		// check for messages
		switch (Event.type)
		{
			// Check if the app has been forced to quit
			case SDL_QUIT:
			{
				_KillFlag = true;
			}
			#ifdef PC
			// check for keypresses
			case SDL_KEYDOWN:
			#endif // PC
			#ifdef GP2X
			case SDL_JOYBUTTONDOWN:
			#endif // GP2X
			{
				#ifdef PC
				switch( Event.key.keysym.sym )
				#endif // PC
				#ifdef GP2X
				switch( Event.jbutton.button )
				#endif // GP2X
				{
					case GP2X_JOY_UP: 			_CurrentInputState |= UP; 					break;
					case GP2X_JOY_DOWN: 		_CurrentInputState |= DOWN; 				break;
					case GP2X_JOY_LEFT: 		_CurrentInputState |= LEFT; 				break;
					case GP2X_JOY_RIGHT: 		_CurrentInputState |= RIGHT; 				break;
					case GP2X_JOY_UPLEFT: 		_CurrentInputState |= UPLEFT;				break;
					case GP2X_JOY_UPRIGHT: 		_CurrentInputState |= UPRIGHT; 				break;
					case GP2X_JOY_DOWNLEFT: 	_CurrentInputState |= DOWNLEFT; 			break;
					case GP2X_JOY_DOWNRIGHT: 	_CurrentInputState |= DOWNRIGHT; 			break;
					case GP2X_JOY_CLICK:		_CurrentInputState |= BUTTON_JOYCLICK; 		break;
					case GP2X_BUTTON_A:			_CurrentInputState |= BUTTON_A;				break;
					case GP2X_BUTTON_B:			_CurrentInputState |= BUTTON_B;				break;
					case GP2X_BUTTON_X:			_CurrentInputState |= BUTTON_X;				break;
					case GP2X_BUTTON_Y:			_CurrentInputState |= BUTTON_Y;				break;
					case GP2X_BUTTON_L:			_CurrentInputState |= BUTTON_L;				break;
					case GP2X_BUTTON_R:			_CurrentInputState |= BUTTON_R;				break;
					case GP2X_BUTTON_SELECT:	_CurrentInputState |= BUTTON_SELECT;		break;
					case GP2X_BUTTON_START:		_CurrentInputState |= BUTTON_START;			break;
					case GP2X_BUTTON_VOLUP:		_CurrentInputState |= BUTTON_VOLUP;			break;
					case GP2X_BUTTON_VOLDOWN:	_CurrentInputState |= BUTTON_VOLDOWN;		break;
					default: break;
				}
			} break;

			#ifdef PC
			case SDL_KEYUP:
			#endif // PC
			#ifdef GP2X
			case SDL_JOYBUTTONUP:
			#endif // GP2X
			{
				#ifdef PC
				switch( Event.key.keysym.sym )
				#endif // PC
				#ifdef GP2X
				switch( Event.jbutton.button )
				#endif // GP2X
				{
					case GP2X_JOY_UP: 			_CurrentInputState &= ~UP; 					break;
					case GP2X_JOY_DOWN: 		_CurrentInputState &= ~DOWN; 				break;
					case GP2X_JOY_LEFT: 		_CurrentInputState &= ~LEFT; 				break;
					case GP2X_JOY_RIGHT: 		_CurrentInputState &= ~RIGHT; 				break;
					case GP2X_JOY_UPLEFT: 		_CurrentInputState &= ~UPLEFT;				break;
					case GP2X_JOY_UPRIGHT: 		_CurrentInputState &= ~UPRIGHT; 			break;
					case GP2X_JOY_DOWNLEFT: 	_CurrentInputState &= ~DOWNLEFT; 			break;
					case GP2X_JOY_DOWNRIGHT: 	_CurrentInputState &= ~DOWNRIGHT; 			break;
					case GP2X_JOY_CLICK:		_CurrentInputState &= ~BUTTON_JOYCLICK; 	break;
					case GP2X_BUTTON_A:			_CurrentInputState &= ~BUTTON_A;			break;
					case GP2X_BUTTON_B:			_CurrentInputState &= ~BUTTON_B;			break;
					case GP2X_BUTTON_X:			_CurrentInputState &= ~BUTTON_X;			break;
					case GP2X_BUTTON_Y:			_CurrentInputState &= ~BUTTON_Y;			break;
					case GP2X_BUTTON_L:			_CurrentInputState &= ~BUTTON_L;			break;
					case GP2X_BUTTON_R:			_CurrentInputState &= ~BUTTON_R;			break;
					case GP2X_BUTTON_SELECT:	_CurrentInputState &= ~BUTTON_SELECT;		break;
					case GP2X_BUTTON_START:		_CurrentInputState &= ~BUTTON_START;		break;
					case GP2X_BUTTON_VOLUP:		_CurrentInputState &= ~BUTTON_VOLUP;		break;
					case GP2X_BUTTON_VOLDOWN:	_CurrentInputState &= ~BUTTON_VOLDOWN;		break;
					default: break;
				}
			} break;
			default: break;
		} // end switch

		// Add in case 5 DaveC situation. If any of the orthogonal directions are pressed clear the diagonals
		if(_CurrentInputState & JOY_ORTHOGONAL)
		{
			_CurrentInputState &= ~JOY_DIAGONALS;
		}

	} // end of message processing
}

/*
History
=======
2006-07-19: Adding support for case 5
2006-07-17: Added a kill app flag and function
2006-06-11: Created file and started the Input Module
*/
